"""
yamelish turns JSON data into a YAML-ish format suitabe for diff operations.
"""

EOL = '\n'
INDENT = '  '

def indent(block):
	"""Indent the given block."""
	lines = block.splitlines()
	indented_lines = [INDENT + line for line in lines]
	return EOL.join(indented_lines)

def handle_value(data):
	"""
	Take a non-string scalar value and turns it into yamelish.
	Return a tuple: yamelish (string), whether this string is a block (boolean)
	"""
	return str(data), False # Never a block

def handle_string(data):
	"""
	Take a string value and turns it into yamelish.
	Return a tuple: yamelish (string), whether this string is a block (boolean)
	"""
	if not data:
		return '""', False # Not a block
	return data, len(data.splitlines()) > 1 # Potentially a block

def handle_object(data):
	"""
	Take a dict (or a dict-like object) and turns it into yamelish.
	Return a tuple: yamelish (string), whether this string is a block (boolean)
	"""
	if not data:
		return '{}', False # Not a block
	string = ''
	for i, key in enumerate(data.keys()):
		if i:
			string += EOL
		string += key + ':'
		s_value, s_block = handle(data[key])
		if s_block:
			string += EOL + indent(s_value)
		else:
			string += ' ' + s_value
	return string, True # Definitely a block

def handle_array(data):
	"""
	Take a list (or a list-like object) and turns it into yamelish.
	Return a tuple: yamelish (string), whether this string is a block (boolean)
	"""
	if not data:
		return '[]', False # Not a block
	string = ''
	for i, value in enumerate(data):
		if i:
			string += EOL
		s_value, s_block = handle(value)
		if s_block:
			string += '-' + indent(s_value)[1:]
		else:
			string += '- ' + s_value
	return string, True # Definitely a block

def handle(data):
	"""
	Take any value and turns it into yamelish.
	Return a tuple: yamelish (string), whether this string is a block (boolean)
	"""
	handlers = {dict: handle_object, list: handle_array, str: handle_string}
	return handlers.get(type(data), handle_value)(data)



"""
Provide a simple Command-Line Interface (CLI) to convert JSON files into yamelish.
"""
import json
import os
import sys
from .handlers import handle, EOL

DECODING = os.environ.get('YAMELISH_DECODING', 'utf-8')
ENCODING = os.environ.get('YAMELISH_ENCODING', 'utf-8')

def load_json(filepath, encoding):
	"""Simple wrapper around open() and json.load()."""
	with open(filepath, encoding=encoding) as filedesc:
		return json.load(filedesc)

def write_to_stdout(string, encoding):
	"""Simple print()-like helper."""
	if string:
		if string[-1] != EOL:
			string += EOL
		sys.stdout.buffer.write(string.encode(encoding))

def run_cli():
	"""
	Take 0 to n JSON files as command-line arguments, turn them into yamelish
	and dump the whole thing on stdout.
	"""
	for filepath in sys.argv[1:]:
		json_data = load_json(filepath, DECODING)
		yamelish = handle(json_data)[0]
		write_to_stdout(yamelish, ENCODING)

if __name__ == '__main__':
	run_cli()



#!/usr/bin/env python3

from setuptools import setup

with open('README.md', 'r') as readme:
	long_description = readme.read()

setup(
	name='yamelish',
	version='1.0.0',
	description='YAML-ish, non parsable format, suitable to diff JSON data',
	long_description=long_description,
	long_description_content_type='text/markdown',
	author='Xavier G.',
	author_email='xavier.yamelish@kindwolf.org',
	url='https://github.com/xavierog/yamelish',
	packages=['yamelish'],
	classifiers=[
		'Operating System :: POSIX :: Linux', # yamelish was never tested outside Linux
		'Programming Language :: Python :: 3 :: Only', # Implemented and tested with Python 3.6; unlikely to run with Python 2.x
		'Topic :: Text Processing',
	],
	license='WTFPL',
	keywords=['cli', 'json', 'yaml', 'diff'],
	package_dir={'': 'src'},
	install_requires=[],
	entry_points={
		'console_scripts': [
			'yamelish = yamelish.cli:run_cli'
		]
	},

)
